

<<CODE
 This is a verbatim block.

 Tags are not recognized here.

 Empty lines do not terminate the paragraph.
 Anything could be placed within, even if it has
 special meanings otherwhere.

 A verbatim block is structured like a "here document":
 a user declared string terminates it.

CODE


<<SECOND_VERBATIM_PARAGRAPH

Just another example. Look: verbatim text can start
at the beginning of a line! (Of course, the declared
termination string should be avoided until the real
point of completion:

SECOND_VERBATIM_PARAGRAPH