
// include macro definitions shared by all basic tag docs
\INCLUDE{type=pp file="basic-tag-macros.pp" smart=1}



=C

The \B<\\C> tag marks text to be \I<\B<c>ode>.


\B<Syntax>

\\C<body>


\B<Options>

No options are allowed. Using options will cause a syntax error.


\B<Body>

The body is the string to be marked.


\B<Notes>

\B<\\C> is part of the \I<basic tag set> supported by \I<all> PerlPoint translators.


\B<Example>

The PerlPoint phrase "This text is \\C<formatted>." is translated into
"This text is \C<formatted>.".


\B<See also>

More basic set tags: \OTHER_BASIC_TAGS{current=C}.

