
// helper macro
+BC:\B<\C<__body__>>


=A first chapter

Text to fill space. Occassionally.

Text to fill space. Occassionally.

==Details 1

Text to fill space. Occassionally.

==Details 2

==Details 3

=Back to start level

Text to fill space. Occassionally.

=Another topic

Text to fill space. Occassionally.

==Subtopic 1

Text to fill space. Occassionally.

==Subtopic 2

===Subtopic 2.1

===Subtopic 2.2

====And more: 2.2.1

Text to fill space. Occassionally.

==\I<Formatted> headline

==And \B<\I<even>> heavier \BC<tag usage>

=Appendix

Text to fill space. Occassionally.

