
Conditions allow to maintain all versions of a presentation in one file.

? $language eq 'German'

Bedingungen erlauben es, veschiedensprachige Versionen einer Präsentation
zusammen in einer Vorlage zu pflegen.

? 1

Back to main text.