
// include macro definitions shared by all basic tag docs
\INCLUDE{type=pp file="basic-tag-macros.pp" smart=1}



=I

The \B<\\I> tag marks text to be formatted \I<\B<i>talic>.


\B<Syntax>

\\I<body>


\B<Options>

No options are allowed. Using options will cause a syntax error.


\B<Body>

The body is the string to be formatted.


\B<Notes>

\B<\\I> is part of the \I<basic tag set> supported by \I<all> PerlPoint translators.


\B<Example>

  This text is \\I<formatted>.

produces

  This text is \I<formatted>.


\B<See also>

More basic set tags: \OTHER_BASIC_TAGS{current=I}.

